@@ -120,10 +120,6 @@
 /*
  * Some systems lack full limit definitions.
  */
-#if defined(HAVE_DECL_OPEN_MAX) && !HAVE_DECL_OPEN_MAX
-# define OPEN_MAX	256
-#endif
-
 #if defined(HAVE_DECL_LLONG_MAX) && !HAVE_DECL_LLONG_MAX
 # if defined(HAVE_DECL_QUAD_MAX) && HAVE_DECL_QUAD_MAX
 #  define LLONG_MAX	QUAD_MAX
@@ -164,14 +160,6 @@
 # endif
 #endif
 
-#if defined(HAVE_DECL_HOST_NAME_MAX) && !HAVE_DECL_HOST_NAME_MAX
-# if defined(HAVE_DECL__POSIX_HOST_NAME_MAX) && HAVE_DECL__POSIX_HOST_NAME_MAX
-#  define HOST_NAME_MAX		_POSIX_HOST_NAME_MAX
-# else
-#  define HOST_NAME_MAX		255
-# endif
-#endif
-
 /*
  * Posix versions for those without...
  */
@@ -201,6 +189,19 @@
 # define S_IRWXU		0000700		/* rwx for owner */
 #endif /* S_IRWXU */
 
+/* For futimens() and utimensat() emulation. */
+#if !defined(HAVE_FUTIMENS) && !defined(HAVE_UTIMENSAT)
+# ifndef UTIME_OMIT
+#  define UTIME_OMIT	-1L
+# endif
+# ifndef UTIME_NOW
+#  define UTIME_NOW	-2L
+# endif
+# ifndef AT_FDCWD
+#  define AT_FDCWD	-100
+# endif
+#endif
+
 /*
  * These should be defined in <unistd.h> but not everyone has them.
  */
@@ -288,14 +289,6 @@ typedef struct sigaction sigaction_t;
 # define HAVE_DIRFD
 #endif
 
-/*
- * Define futimes() in terms of futimesat() if needed.
- */
-#if !defined(HAVE_FUTIMES) && defined(HAVE_FUTIMESAT)
-# define futimes(_f, _tv)	futimesat(_f, NULL, _tv)
-# define HAVE_FUTIMES
-#endif
-
 #if !defined(HAVE_KILLPG) && !defined(killpg)
 # define killpg(s)	kill(-(s))
 #endif
@@ -398,16 +391,16 @@ __dso_public ssize_t sudo_getline(char **bufp, size_t *bufsizep, FILE *fp);
 # undef getline
 # define getline(_a, _b, _c) sudo_getline((_a), (_b), (_c))
 #endif /* HAVE_GETLINE */
-#ifndef HAVE_UTIMES
-__dso_public int sudo_utimes(const char *file, const struct timeval *times);
-# undef utimes
-# define utimes(_a, _b) sudo_utimes((_a), (_b))
-#endif /* HAVE_UTIMES */
-#ifdef HAVE_FUTIME
-__dso_public int sudo_futimes(int fd, const struct timeval *times);
-# undef futimes
-# define futimes(_a, _b) sudo_futimes((_a), (_b))
-#endif /* HAVE_FUTIME */
+#ifndef HAVE_UTIMENSAT
+__dso_public int sudo_utimensat(int fd, const char *file, const struct timespec *times, int flag);
+# undef utimensat
+# define utimensat(_a, _b, _c, _d) sudo_utimensat((_a), (_b), (_c), (_d))
+#endif /* HAVE_UTIMENSAT */
+#ifndef HAVE_FUTIMENS
+__dso_public int sudo_futimens(int fd, const struct timespec *times);
+# undef futimens
+# define futimens(_a, _b) sudo_futimens((_a), (_b))
+#endif /* HAVE_FUTIMENS */
 #if !defined(HAVE_SNPRINTF) || defined(PREFER_PORTABLE_SNPRINTF)
 __dso_public int sudo_snprintf(char *str, size_t n, char const *fmt, ...) __printflike(3, 4);
 # undef snprintf
@@ -471,15 +464,6 @@ __dso_public int sudo_sig2str(int signo, char *signame);
 # undef sig2str
 # define sig2str(_a, _b) sudo_sig2str((_a), (_b))
 #endif /* HAVE_SIG2STR */
-#ifndef HAVE_CLOCK_GETTIME
-# define CLOCK_REALTIME 0
-# ifdef __MACH__
-#  define CLOCK_MONOTONIC 1
-# endif
-__dso_public int sudo_clock_gettime(clockid_t clock_id, struct timespec *tp);
-# undef clock_gettime
-# define clock_gettime(_a, _b) sudo_clock_gettime((_a), (_b))
-#endif /* HAVE_CLOCK_GETTIME */
 #if !defined(HAVE_INET_NTOP) && defined(_SUDO_NET_IFS_C)
 __dso_public char *sudo_inet_ntop(int af, const void *src, char *dst, socklen_t size);
 # undef inet_ntop
