@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005,2008,2010-2013 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2005,2008,2010-2015 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -44,13 +44,9 @@
 #include <pwd.h>
 #include <grp.h>
 
-#include "missing.h"
+#include "sudo_compat.h"
 #include "sudo_util.h"
 
-#ifndef LINE_MAX
-# define LINE_MAX 2048
-#endif
-
 #undef GRMEM_MAX
 #define GRMEM_MAX 200
 
@@ -123,7 +119,7 @@ next_entry:
     if ((colon = strchr(cp = colon, ':')) == NULL)
 	goto next_entry;
     *colon++ = '\0';
-    id = atoid(cp, NULL, NULL, &errstr);
+    id = sudo_strtoid(cp, NULL, NULL, &errstr);
     if (errstr != NULL)
 	goto next_entry;
     gr.gr_gid = (gid_t)id;
