@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2005, 2007-2014
+ * Copyright (c) 2000-2005, 2007-2015
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -41,7 +41,9 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif /* HAVE_UNISTD_H */
-#ifdef HAVE_INTTYPES_H
+#if defined(HAVE_STDINT_H)
+# include <stdint.h>
+#elif defined(HAVE_INTTYPES_H)
 # include <inttypes.h>
 #endif
 #ifdef HAVE_LOGIN_CAP_H
@@ -99,8 +101,8 @@
 #define KEPT_MAX    	0xff00
 
 struct environment {
-    char * const *old_envp;	/* pointer the environment we passed back */
     char **envp;		/* pointer to the new environment */
+    char **old_envp;		/* pointer the old environment we allocated */
     size_t env_size;		/* size of new_environ in char **'s */
     size_t env_len;		/* number of slots used, not counting NULL */
 };
@@ -170,6 +172,8 @@ static const char *initial_badenv_table[] = {
     "PYTHONUSERBASE",		/* python, per user site-packages directory */
     "RUBYLIB",			/* ruby, library load path */
     "RUBYOPT",			/* ruby, extra command line options */
+    "BASH_FUNC_*",		/* new-style bash functions */
+    "__BASH_FUNC<*",		/* new-style bash functions (Apple) */
     NULL
 };
 
@@ -183,6 +187,7 @@ static const char *initial_checkenv_table[] = {
     "LC_*",
     "LINGUAS",
     "TERM",
+    "TZ",
     NULL
 };
 
@@ -198,7 +203,6 @@ static const char *initial_keepenv_table[] = {
     "PATH",
     "PS1",
     "PS2",
-    "TZ",
     "XAUTHORITY",
     "XAUTHORIZATION",
     NULL
@@ -212,13 +216,17 @@ env_init(char * const envp[])
 {
     char * const *ep;
     size_t len;
-    debug_decl(env_init, SUDO_DEBUG_ENV)
+    debug_decl(env_init, SUDOERS_DEBUG_ENV)
 
     if (envp == NULL) {
+	/* Free the old envp we allocated, if any. */
+	sudo_efree(env.old_envp);
+
 	/* Reset to initial state but keep a pointer to what we allocated. */
-	envp = env.envp;
-	memset(&env, 0, sizeof(env));
-	env.old_envp = envp;
+	env.old_envp = env.envp;
+	env.envp = NULL;
+	env.env_size = 0;
+	env.env_len = 0;
     } else {
 	/* Make private copy of envp. */
 	for (ep = envp; *ep != NULL; ep++)
@@ -235,8 +243,8 @@ env_init(char * const envp[])
 	env.envp[len] = NULL;
 
 	/* Free the old envp we allocated, if any. */
-	if (env.old_envp != NULL)
-	    sudo_efree((void *)env.old_envp);
+	sudo_efree(env.old_envp);
+	env.old_envp = NULL;
     }
 
     debug_return;
@@ -251,6 +259,22 @@ env_get(void)
     return env.envp;
 }
 
+/*
+ * Swap the old and new copies of the environment.
+ */
+bool
+env_swap_old(void)
+{
+    char **old_envp;
+
+    if (env.old_envp == NULL)
+	return false;
+    old_envp = env.old_envp;
+    env.old_envp = env.envp;
+    env.envp = old_envp;
+    return true;
+}
+
 /*
  * Similar to putenv(3) but operates on sudo's private copy of the
  * environment (not environ) and it always overwrites.  The dupcheck param
@@ -346,7 +370,7 @@ static int
 sudo_putenv(char *str, bool dupcheck, bool overwrite)
 {
     int rval;
-    debug_decl(sudo_putenv, SUDO_DEBUG_ENV)
+    debug_decl(sudo_putenv, SUDOERS_DEBUG_ENV)
 
     sudo_debug_printf(SUDO_DEBUG_INFO, "sudo_putenv: %s", str);
 
@@ -371,7 +395,7 @@ sudo_setenv2(const char *var, const char *val, bool dupcheck, bool overwrite)
     char *estring;
     size_t esize;
     int rval = -1;
-    debug_decl(sudo_setenv2, SUDO_DEBUG_ENV)
+    debug_decl(sudo_setenv2, SUDOERS_DEBUG_ENV)
 
     esize = strlen(var) + 1 + strlen(val) + 1;
     estring = sudo_emalloc(esize);
@@ -486,7 +510,7 @@ int
 sudo_unsetenv(const char *name)
 {
     int rval;
-    debug_decl(sudo_unsetenv, SUDO_DEBUG_ENV)
+    debug_decl(sudo_unsetenv, SUDOERS_DEBUG_ENV)
 
     sudo_debug_printf(SUDO_DEBUG_INFO, "sudo_unsetenv: %s", name);
 
@@ -526,7 +550,7 @@ char *
 sudo_getenv(const char *name)
 {
     char *val;
-    debug_decl(sudo_getenv, SUDO_DEBUG_ENV)
+    debug_decl(sudo_getenv, SUDOERS_DEBUG_ENV)
 
     sudo_debug_printf(SUDO_DEBUG_INFO, "sudo_getenv: %s", name);
 
@@ -544,7 +568,7 @@ matches_env_list(const char *var, struct list_members *list, bool *full_match)
 {
     struct list_member *cur;
     bool match = false;
-    debug_decl(matches_env_list, SUDO_DEBUG_ENV)
+    debug_decl(matches_env_list, SUDOERS_DEBUG_ENV)
 
     SLIST_FOREACH(cur, list, entries) {
 	size_t sep_pos, len = strlen(cur->value);
@@ -577,12 +601,60 @@ static bool
 matches_env_delete(const char *var)
 {
     bool full_match;	/* unused */
-    debug_decl(matches_env_delete, SUDO_DEBUG_ENV)
+    debug_decl(matches_env_delete, SUDOERS_DEBUG_ENV)
 
     /* Skip anything listed in env_delete. */
     debug_return_bool(matches_env_list(var, &def_env_delete, &full_match));
 }
 
+/*
+ * Sanity-check the TZ environment variable.
+ * On many systems it is possible to set this to a pathname.
+ */
+static bool
+tz_is_sane(const char *tzval)
+{
+    const char *cp;
+    char lastch;
+    debug_decl(tz_is_sane, SUDOERS_DEBUG_ENV)
+
+    /* tzcode treats a value beginning with a ':' as a path. */
+    if (tzval[0] == ':')
+	tzval++;
+
+    /* Reject fully-qualified TZ that doesn't being with the zoneinfo dir. */
+    if (tzval[0] == '/') {
+#ifdef _PATH_ZONEINFO
+	if (strncmp(tzval, _PATH_ZONEINFO, sizeof(_PATH_ZONEINFO) - 1) != 0 ||
+	    tzval[sizeof(_PATH_ZONEINFO) - 1] != '/')
+	    debug_return_bool(false);
+#else
+	/* Assume the worst. */
+	debug_return_bool(false);
+#endif
+    }
+
+    /*
+     * Make sure TZ only contains printable non-space characters
+     * and does not contain a '..' path element.
+     */
+    lastch = '/';
+    for (cp = tzval; *cp != '\0'; cp++) {
+	if (isspace((unsigned char)*cp) || !isprint((unsigned char)*cp))
+	    debug_return_bool(false);
+	if (lastch == '/' && cp[0] == '.' && cp[1] == '.' &&
+	    (cp[2] == '/' || cp[2] == '\0'))
+	    debug_return_bool(false);
+	lastch = *cp;
+    }
+
+    /* Reject extra long TZ values (even if not a path). */
+    if ((size_t)(cp - tzval) >= PATH_MAX)
+	debug_return_bool(false);
+
+    debug_return_bool(true);
+}
+
 /*
  * Apply the env_check list.
  * Returns true if the variable is allowed, false if denied
@@ -592,13 +664,18 @@ static int
 matches_env_check(const char *var, bool *full_match)
 {
     int keepit = -1;
-    debug_decl(matches_env_check, SUDO_DEBUG_ENV)
+    debug_decl(matches_env_check, SUDOERS_DEBUG_ENV)
 
     /* Skip anything listed in env_check that includes '/' or '%'. */
     if (matches_env_list(var, &def_env_check, full_match)) {
-	const char *val = strchr(var, '=');
-	if (val != NULL)
-	    keepit = !strpbrk(++val, "/%");
+	if (strncmp(var, "TZ=", 3) == 0) {
+	    /* Special case for TZ */
+	    keepit = tz_is_sane(var + 3);
+	} else {
+	    const char *val = strchr(var, '=');
+	    if (val != NULL)
+		keepit = !strpbrk(++val, "/%");
+	}
     }
     debug_return_bool(keepit);
 }
@@ -611,7 +688,7 @@ static bool
 matches_env_keep(const char *var, bool *full_match)
 {
     bool keepit = false;
-    debug_decl(matches_env_keep, SUDO_DEBUG_ENV)
+    debug_decl(matches_env_keep, SUDOERS_DEBUG_ENV)
 
     /* Preserve SHELL variable for "sudo -s". */
     if (ISSET(sudo_mode, MODE_SHELL) && strncmp(var, "SHELL=", 6) == 0) {
@@ -632,7 +709,7 @@ env_should_delete(const char *var)
     const char *cp;
     int delete_it;
     bool full_match = false;
-    debug_decl(env_should_delete, SUDO_DEBUG_ENV);
+    debug_decl(env_should_delete, SUDOERS_DEBUG_ENV);
 
     /* Skip variables with values beginning with () (bash functions) */
     if ((cp = strchr(var, '=')) != NULL) {
@@ -662,7 +739,7 @@ env_should_keep(const char *var)
     int keepit;
     bool full_match = false;
     const char *cp;
-    debug_decl(env_should_keep, SUDO_DEBUG_ENV)
+    debug_decl(env_should_keep, SUDOERS_DEBUG_ENV)
 
     keepit = matches_env_check(var, &full_match);
     if (keepit == -1)
@@ -676,7 +753,7 @@ env_should_keep(const char *var)
 	}
     }
     sudo_debug_printf(SUDO_DEBUG_INFO, "keep %s: %s",
-	var, keepit ? "YES" : "NO");
+	var, keepit == true ? "YES" : "NO");
     debug_return_bool(keepit == true);
 }
 
@@ -692,7 +769,7 @@ env_merge(char * const envp[])
 {
     char * const *ep;
     bool rval = true;
-    debug_decl(env_merge, SUDO_DEBUG_ENV)
+    debug_decl(env_merge, SUDOERS_DEBUG_ENV)
 
     for (ep = envp; *ep != NULL; ep++) {
 	/* XXX - avoid checking value here, should only check name */
@@ -753,11 +830,11 @@ env_update_didvar(const char *ep, unsigned int *didvar)
 bool
 rebuild_env(void)
 {
-    char **old_envp, **ep, *cp, *ps1;
+    char **ep, *cp, *ps1;
     char idbuf[MAX_UID_T_LEN + 1];
     unsigned int didvar;
     bool reset_home = false;
-    debug_decl(rebuild_env, SUDO_DEBUG_ENV)
+    debug_decl(rebuild_env, SUDOERS_DEBUG_ENV)
 
     /*
      * Either clean out the environment or reset to a safe default.
@@ -766,7 +843,8 @@ rebuild_env(void)
     didvar = 0;
     env.env_len = 0;
     env.env_size = 128;
-    old_envp = env.envp;
+    sudo_efree(env.old_envp);
+    env.old_envp = env.envp;
     env.envp = sudo_emallocarray(env.env_size, sizeof(char *));
 #ifdef ENV_DEBUG
     memset(env.envp, 0, env.env_size * sizeof(char *));
@@ -810,7 +888,7 @@ rebuild_env(void)
 	}
 
 	/* Pull in vars we want to keep from the old environment. */
-	for (ep = old_envp; *ep; ep++) {
+	for (ep = env.old_envp; *ep; ep++) {
 	    bool keepit;
 
 	    /*
@@ -886,7 +964,7 @@ rebuild_env(void)
 	 * Copy environ entries as long as they don't match env_delete or
 	 * env_check.
 	 */
-	for (ep = old_envp; *ep; ep++) {
+	for (ep = env.old_envp; *ep; ep++) {
 	    /* Add variable unless it matches a black list. */
 	    if (!env_should_delete(*ep)) {
 		if (strncmp(*ep, "SUDO_PS1=", 9) == 0)
@@ -913,7 +991,7 @@ rebuild_env(void)
      * they have already been set) or sudoedit (because we want the editor
      * to find the invoking user's startup files).
      */
-    if (def_set_logname && !ISSET(sudo_mode, MODE_LOGIN_SHELL|MODE_EDIT)) {
+    if (def_set_logname && !ISSET(sudo_mode, MODE_LOGIN_SHELL)) {
 	if (!ISSET(didvar, KEPT_LOGNAME))
 	    sudo_setenv2("LOGNAME", runas_pw->pw_name, true, true);
 	if (!ISSET(didvar, KEPT_USER))
@@ -964,9 +1042,6 @@ rebuild_env(void)
     if (sudo_setenv2("SUDO_GID", idbuf, true, true) == -1)
 	goto bad;
 
-    /* Free old environment. */
-    sudo_efree(old_envp);
-
     debug_return_bool(true);
 
 bad:
@@ -983,7 +1058,7 @@ insert_env_vars(char * const envp[])
 {
     char * const *ep;
     bool rval = true;
-    debug_decl(insert_env_vars, SUDO_DEBUG_ENV)
+    debug_decl(insert_env_vars, SUDOERS_DEBUG_ENV)
 
     /* Add user-specified environment variables. */
     if (envp != NULL) {
@@ -1011,7 +1086,7 @@ validate_env_vars(char * const env_vars[])
     char *eq, *bad = NULL;
     size_t len, blen = 0, bsize = 0;
     bool okvar, rval = true;
-    debug_decl(validate_env_vars, SUDO_DEBUG_ENV)
+    debug_decl(validate_env_vars, SUDOERS_DEBUG_ENV)
 
     if (env_vars == NULL)
 	debug_return_bool(true);	/* nothing to do */
@@ -1073,7 +1148,7 @@ read_env_file(const char *path, int overwrite)
     bool rval = true;
     char *cp, *var, *val, *line = NULL;
     size_t var_len, val_len, linesize = 0;
-    debug_decl(read_env_file, SUDO_DEBUG_ENV)
+    debug_decl(read_env_file, SUDOERS_DEBUG_ENV)
 
     if ((fp = fopen(path, "r")) == NULL) {
 	if (errno != ENOENT)
