@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2014-2015 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -51,15 +51,6 @@
 #include "sudoers.h"
 #include "check.h"
 
-/* On Linux, CLOCK_MONOTONIC does not run while suspended. */
-#if defined(CLOCK_BOOTTIME)
-# define SUDO_CLOCK_MONOTONIC	CLOCK_BOOTTIME
-#elif defined(CLOCK_MONOTONIC)
-# define SUDO_CLOCK_MONOTONIC	CLOCK_MONOTONIC
-#else
-# define SUDO_CLOCK_MONOTONIC	CLOCK_REALTIME
-#endif
-
 static char timestamp_file[PATH_MAX];
 static off_t timestamp_hint = (off_t)-1;
 static struct timestamp_entry timestamp_key;
@@ -70,7 +61,7 @@ static struct timestamp_entry timestamp_key;
 static bool
 ts_match_record(struct timestamp_entry *key, struct timestamp_entry *entry)
 {
-    debug_decl(ts_match_record, SUDO_DEBUG_AUTH)
+    debug_decl(ts_match_record, SUDOERS_DEBUG_AUTH)
 
     if (entry->version != key->version)
 	debug_return_bool(false);
@@ -110,7 +101,7 @@ static bool
 ts_find_record(int fd, struct timestamp_entry *key, struct timestamp_entry *entry)
 {
     struct timestamp_entry cur;
-    debug_decl(ts_find_record, SUDO_DEBUG_AUTH)
+    debug_decl(ts_find_record, SUDOERS_DEBUG_AUTH)
 
     /*
      * Look for a matching record.
@@ -145,7 +136,7 @@ ts_update_record(int fd, struct timestamp_entry *entry, off_t timestamp_hint)
     struct timestamp_entry cur;
     ssize_t nwritten;
     off_t old_eof = (off_t)-1;
-    debug_decl(ts_update_record, SUDO_DEBUG_AUTH)
+    debug_decl(ts_update_record, SUDOERS_DEBUG_AUTH)
 
     /* First try the hint if one is given. */
     if (timestamp_hint != (off_t)-1) {
@@ -215,7 +206,7 @@ ts_mkdirs(char *path, uid_t owner, mode_t mode, mode_t parent_mode, bool quiet)
     gid_t parent_gid = 0;
     char *slash = path;
     bool rval = false;
-    debug_decl(ts_mkdirs, SUDO_DEBUG_AUTH)
+    debug_decl(ts_mkdirs, SUDOERS_DEBUG_AUTH)
 
     while ((slash = strchr(slash + 1, '/')) != NULL) {
 	*slash = '\0';
@@ -265,7 +256,7 @@ ts_secure_dir(char *path, bool make_it, bool quiet)
 {
     struct stat sb;
     bool rval = false;
-    debug_decl(ts_secure_dir, SUDO_DEBUG_AUTH)
+    debug_decl(ts_secure_dir, SUDOERS_DEBUG_AUTH)
 
     sudo_debug_printf(SUDO_DEBUG_INFO|SUDO_DEBUG_LINENO, "checking %s", path);
     switch (sudo_secure_dir(path, timestamp_uid, -1, &sb)) {
@@ -309,7 +300,7 @@ int
 build_timestamp(struct passwd *pw)
 {
     int len;
-    debug_decl(build_timestamp, SUDO_DEBUG_AUTH)
+    debug_decl(build_timestamp, SUDOERS_DEBUG_AUTH)
 
     len = snprintf(timestamp_file, sizeof(timestamp_file), "%s/%s",
 	def_timestampdir, user_name);
@@ -333,7 +324,7 @@ update_timestamp(struct passwd *pw)
     bool uid_changed = false;
     bool rval = false;
     int fd;
-    debug_decl(update_timestamp, SUDO_DEBUG_AUTH)
+    debug_decl(update_timestamp, SUDOERS_DEBUG_AUTH)
 
     /* Zero timeout means don't update the time stamp file. */
     if (def_timestamp_timeout == 0)
@@ -345,7 +336,10 @@ update_timestamp(struct passwd *pw)
 
     /* Fill in time stamp. */
     memcpy(&entry, &timestamp_key, sizeof(struct timestamp_entry));
-    clock_gettime(SUDO_CLOCK_MONOTONIC, &entry.ts);
+    if (sudo_gettime_mono(&entry.ts) == -1) {
+	log_warning(0, N_("unable to read the clock"));
+	goto done;
+    }
 
     /* Open time stamp file and lock it for exclusive access. */
     if (timestamp_uid != 0)
@@ -382,7 +376,7 @@ timestamp_status(struct passwd *pw)
     int status = TS_ERROR;		/* assume the worst */
     struct stat sb;
     int fd = -1;
-    debug_decl(timestamp_status, SUDO_DEBUG_AUTH)
+    debug_decl(timestamp_status, SUDOERS_DEBUG_AUTH)
 
     /* Reset time stamp offset hint. */
     timestamp_hint = (off_t)-1;
@@ -427,7 +421,10 @@ timestamp_status(struct passwd *pw)
 	    timestamp_key.u.ppid = getppid();
 	}
     }
-    clock_gettime(SUDO_CLOCK_MONOTONIC, &timestamp_key.ts);
+    if (sudo_gettime_mono(&timestamp_key.ts) == -1) {
+	log_warning(0, N_("unable to read the clock"));
+	status = TS_ERROR;
+    }
 
     /* If the time stamp dir is missing there is nothing to do. */
     if (status == TS_MISSING)
@@ -447,10 +444,10 @@ timestamp_status(struct passwd *pw)
 
     /* Ignore and clear time stamp file if mtime predates boot time. */
     if (fstat(fd, &sb) == 0) {
-	struct timeval boottime, mtime;
+	struct timespec boottime, mtime;
 
-	mtim_get(&sb, &mtime);
-	if (get_boottime(&boottime) && sudo_timevalcmp(&mtime, &boottime, <)) {
+	mtim_get(&sb, mtime);
+	if (get_boottime(&boottime) && sudo_timespeccmp(&mtime, &boottime, <)) {
 	    ignore_result(ftruncate(fd, (off_t)0));
 	    status = TS_MISSING;
 	    goto done;
@@ -533,7 +530,7 @@ remove_timestamp(bool unlink_it)
     struct timestamp_entry entry;
     bool uid_changed = false;
     int fd = -1;
-    debug_decl(remove_timestamp, SUDO_DEBUG_AUTH)
+    debug_decl(remove_timestamp, SUDOERS_DEBUG_AUTH)
 
     if (build_timestamp(NULL) == -1)
 	debug_return;
@@ -602,7 +599,7 @@ already_lectured(int unused)
     char status_file[PATH_MAX];
     struct stat sb;
     int len;
-    debug_decl(already_lectured, SUDO_DEBUG_AUTH)
+    debug_decl(already_lectured, SUDOERS_DEBUG_AUTH)
 
     if (ts_secure_dir(def_lecture_status_dir, false, true)) {
 	len = snprintf(status_file, sizeof(status_file), "%s/%s",
@@ -626,7 +623,7 @@ set_lectured(void)
     char lecture_status[PATH_MAX];
     bool uid_changed = false;
     int len, fd = -1;
-    debug_decl(set_lectured, SUDO_DEBUG_AUTH)
+    debug_decl(set_lectured, SUDOERS_DEBUG_AUTH)
 
     len = snprintf(lecture_status, sizeof(lecture_status), "%s/%s",
 	def_lecture_status_dir, user_name);
