@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2008, 2010-2013
+ * Copyright (c) 2005, 2008, 2010-2015
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -49,10 +49,6 @@
 #include "tsgetgrpw.h"
 #include "sudoers.h"
 
-#ifndef LINE_MAX
-# define LINE_MAX 2048
-#endif
-
 #undef GRMEM_MAX
 #define GRMEM_MAX 200
 
