@@ -70,6 +70,9 @@
 #ifdef TIME_WITH_SYS_TIME
 # include <time.h>
 #endif
+#ifndef HAVE_STRUCT_TIMESPEC
+# include "compat/timespec.h"
+#endif
 
 #include "sudoers.h"
 #include "parse.h"
@@ -321,8 +324,8 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
     char *cp;				/* scratch char pointer */
     char buf[PATH_MAX*2];		/* buffer used for copying files */
     char linestr[64];			/* string version of lineno */
-    struct timeval tv, times[2];	/* time before and after edit */
-    struct timeval orig_mtim;		/* starting mtime of sudoers file */
+    struct timespec ts, times[2];	/* time before and after edit */
+    struct timespec orig_mtim;		/* starting mtime of sudoers file */
     off_t orig_size;			/* starting size of sudoers file */
     ssize_t nread;			/* number of bytes read */
     struct stat sb;			/* stat buffer */
@@ -332,7 +335,7 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
     if (fstat(sp->fd, &sb) == -1)
 	sudo_fatal(U_("unable to stat %s"), sp->path);
     orig_size = sb.st_size;
-    mtim_get(&sb, &orig_mtim);
+    mtim_get(&sb, orig_mtim);
 
     /* Create the temp file if needed and set timestamp. */
     if (sp->tpath == NULL) {
@@ -358,8 +361,8 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
 	(void) close(tfd);
     }
     times[0].tv_sec = times[1].tv_sec = orig_mtim.tv_sec;
-    times[0].tv_usec = times[1].tv_usec = orig_mtim.tv_usec;
-    (void) utimes(sp->tpath, times);
+    times[0].tv_nsec = times[1].tv_nsec = orig_mtim.tv_nsec;
+    (void) utimensat(AT_FDCWD, sp->tpath, times, 0);
 
     /* Does the editor support +lineno? */
     if (lineno > 0)
@@ -429,9 +432,16 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
      *  XPG4 specifies that vi's exit value is a function of the
      *  number of errors during editing (?!?!).
      */
-    gettimeofday(&times[0], NULL);
+    if (sudo_gettime_real(&times[0]) == -1) {
+	sudo_warn(U_("unable to read the clock"));
+	goto done;
+    }
+
     if (run_command(editor, av) != -1) {
-	gettimeofday(&times[1], NULL);
+	if (sudo_gettime_real(&times[1]) == -1) {
+	    sudo_warn(U_("unable to read the clock"));
+	    goto done;
+	}
 	/*
 	 * Sanity checks.
 	 */
@@ -453,13 +463,13 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
 
     /* Set modified bit if the user changed the file. */
     modified = true;
-    mtim_get(&sb, &tv);
-    if (orig_size == sb.st_size && sudo_timevalcmp(&orig_mtim, &tv, ==)) {
+    mtim_get(&sb, ts);
+    if (orig_size == sb.st_size && sudo_timespeccmp(&orig_mtim, &ts, ==)) {
 	/*
 	 * If mtime and size match but the user spent no measurable
 	 * time in the editor we can't tell if the file was changed.
 	 */
-	if (sudo_timevalcmp(&times[0], &times[1], !=))
+	if (sudo_timespeccmp(&times[0], &times[1], !=))
 	    modified = false;
     }
 
@@ -613,7 +623,7 @@ install_sudoers(struct sudoersfile *sp, bool oldperms)
 	}
 	if (chmod(sp->tpath, sudoers_mode) != 0) {
 	    sudo_warn(U_("unable to change mode of %s to 0%o"), sp->tpath,
-		sudoers_mode);
+		(unsigned int)sudoers_mode);
 	    goto done;
 	}
     }
@@ -811,7 +821,7 @@ check_owner(const char *path, bool quiet)
 	    ok = false;
 	    if (!quiet) {
 		fprintf(stderr, _("%s: bad permissions, should be mode 0%o\n"),
-		    path, sudoers_mode);
+		    path, (unsigned int)sudoers_mode);
 	    }
 	}
     }
@@ -1062,13 +1072,10 @@ get_args(char *cmnd)
 static void
 get_hostname(void)
 {
-    char *p, thost[HOST_NAME_MAX + 1];
+    char *p;
     debug_decl(get_hostname, SUDOERS_DEBUG_UTIL)
 
-    if (gethostname(thost, sizeof(thost)) != -1) {
-	thost[sizeof(thost) - 1] = '\0';
-	user_host = sudo_estrdup(thost);
-
+    if ((user_host = sudo_gethostname()) != NULL) {
 	if ((p = strchr(user_host, '.'))) {
 	    *p = '\0';
 	    user_shost = sudo_estrdup(user_host);
