@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2014 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2013-2015 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,6 +18,8 @@
 
 #include <sys/types.h>
 
+#if !defined(HAVE_SYS_WEAK_ALIAS)
+
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_STRING_H
@@ -48,3 +50,5 @@ sudo_warn_strerror_v1(int errnum)
 {
     return strerror(errnum);
 }
+
+#endif /* !HAVE_SYS_WEAK_ALIAS */
