@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2010-2014 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2008, 2010-2015 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -77,8 +77,10 @@ main(int argc, char *argv[], char *envp[])
     if (argc < 2)
 	sudo_fatalx(U_("requires at least one argument"));
 
-    /* Read sudo.conf. */
-    sudo_conf_read(NULL);
+    /* Read sudo.conf and initialize the debug subsystem. */
+    sudo_conf_read(NULL, SUDO_CONF_DEBUG);
+    sudo_debug_register(getprogname(), NULL, NULL,
+	sudo_conf_debug_files(getprogname()));
 
     if (strcmp(argv[1], "-e") == 0) {
 	ret = sesh_sudoedit(argc, argv);
@@ -119,9 +121,9 @@ sesh_sudoedit(int argc, char *argv[])
     int fd_src, fd_dst, i, oflags_dst, post, ret = SESH_ERR_FAILURE;
     ssize_t nread, nwritten;
     struct stat sb;
-    struct timeval times[2];
+    struct timespec times[2];
     char buf[BUFSIZ];
-    debug_decl(sesh_sudoedit, SUDO_DEBUG_MAIN)
+    debug_decl(sesh_sudoedit, SUDO_DEBUG_EDIT)
 
     if (argc < 3)
 	debug_return_int(SESH_ERR_FAILURE);
@@ -204,14 +206,13 @@ sesh_sudoedit(int argc, char *argv[])
 		if (fd_src == -1 || fstat(fd_src, &sb) != 0)
 		    memset(&sb, 0, sizeof(sb));
 		/* Make mtime on temp file match src. */
-		mtim_get(&sb, &times[0]);
+		mtim_get(&sb, times[0]);
 		times[1].tv_sec = times[0].tv_sec;
-		times[1].tv_usec = times[0].tv_usec;
-#ifdef HAVE_FUTIMES
-		(void) futimes(fd_dst, times);
-#else
-	        (void) utimes(path_dst, times);
-#endif
+		times[1].tv_nsec = times[0].tv_nsec;
+		if (futimens(fd_dst, times) == -1) {
+		    if (utimensat(AT_FDCWD, path_dst, times, 0) == -1)
+			sudo_warn("%s", path_dst);
+		}
 	    }
 	    close(fd_dst);
 	}
